@@ -1,7 +1,7 @@
 /*
  * This file is part of the QPackageKit project
  * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
- * Copyright (C) 2010-2012 Daniel Nicoletti <dantti12@gmail.com>
+ * Copyright (C) 2010-2018 Daniel Nicoletti <dantti12@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,6 +27,8 @@
 #include <QtDBus/QDBusObjectPath>
 #include <QtDBus/QDBusPendingReply>
 
+#include <packagekitqt_global.h>
+
 #include "bitfield.h"
 
 namespace PackageKit {
@@ -43,29 +45,14 @@ class Details;
 * A Transaction is created whenever you do an asynchronous action (for example a Search, Install...).
 * This class allows you to monitor and control the flow of the action.
 *
-* You should delete the transaction after finished() is emitted
+* Transaction will be automatically deleted after finished() is emitted
 *
 * \sa Daemon
 */
 class TransactionPrivate;
-class Transaction : public QObject
+class PACKAGEKITQT_LIBRARY Transaction : public QObject
 {
     Q_OBJECT
-    Q_ENUMS(Role)
-    Q_ENUMS(Error)
-    Q_ENUMS(Exit)
-    Q_ENUMS(Message)
-    Q_ENUMS(Status)
-    Q_ENUMS(MediaType)
-    Q_ENUMS(Provides)
-    Q_ENUMS(DistroUpgrade)
-    Q_ENUMS(TransactionFlag)
-    Q_ENUMS(Restart)
-    Q_ENUMS(UpdateState)
-    Q_ENUMS(Group)
-    Q_ENUMS(Info)
-    Q_ENUMS(SigType)
-    Q_ENUMS(Filter)
     Q_FLAGS(TransactionFlag TransactionFlags)
     Q_FLAGS(Filter Filters)
     Q_PROPERTY(QDBusObjectPath tid READ tid)
@@ -143,8 +130,10 @@ public:
         RoleRepairSystem,       // Since 0.7.2
         RoleGetDetailsLocal,    // Since 0.8.17
         RoleGetFilesLocal,      // Since 0.9.1
-        RoleRepoRemove          // Since 0.9.1
+        RoleRepoRemove,         // Since 0.9.1
+        RoleUpgradeSystem       // since 1.0.11
     };
+    Q_ENUM(Role)
     typedef Bitfield Roles;
 
     /**
@@ -220,6 +209,7 @@ public:
         ErrorUnfinishedTransaction,
         ErrorLockRequired
     };
+    Q_ENUM(Error)
 
     /**
      * Describes how the transaction finished
@@ -238,6 +228,7 @@ public:
         ExitCancelledPriority,
         ExitRepairRequired
     };
+    Q_ENUM(Exit)
 
     /**
      * Describes the different package filters
@@ -269,34 +260,14 @@ public:
         FilterNotCollections = 0x0800000,
         FilterApplication    = 0x1000000,
         FilterNotApplication = 0x2000000,
+        FilterDownloaded     = 0x4000000,
+        FilterNotDownloaded  = 0x8000000,
         /* this always has to be at the end of the list */
-        FilterLast           = 0x4000000
+        FilterLast           = 0x10000000
     };
+    Q_ENUM(Filter)
     Q_DECLARE_FLAGS(Filters, Filter)
 
-    /**
-     * Describes a message's type
-     */
-    enum Message {
-        MessageUnknown,
-        MessageBrokenMirror,
-        MessageConnectionRefused,
-        MessageParameterInvalid,
-        MessagePriorityInvalid,
-        MessageBackendError,
-        MessageDaemonError,
-        MessageCacheBeingRebuilt,
-        MessageUntrustedPackage,
-        MessageNewerPackageExists,
-        MessageCouldNotFindPackage,
-        MessageConfigFilesChanged,
-        MessagePackageAlreadyInstalled,
-        MessageAutoremoveIgnored,
-        MessageRepoMetadataDownloadFailed,
-        MessageRepoForDevelopersOnly,
-        MessageOtherUpdatesHeldBack
-    };
-
     /**
      * Describes the current state of the transaction
      */
@@ -336,8 +307,10 @@ public:
         StatusScanProcessList,
         StatusCheckExecutableFiles,
         StatusCheckLibraries,
-        StatusCopyFiles
+        StatusCopyFiles,
+        StatusRunHook
     };
+    Q_ENUM(Status)
 
     /**
      * Describes what kind of media is required
@@ -348,6 +321,7 @@ public:
         MediaTypeDvd,
         MediaTypeDisc
     };
+    Q_ENUM(MediaType)
 
     /**
      * Describes an distro upgrade state
@@ -357,18 +331,35 @@ public:
         DistroUpgradeStable,
         DistroUpgradeUnstable
     };
+    Q_ENUM(DistroUpgrade)
 
     /**
      * Describes the type of distribution upgrade to perform
-     * \sa upgradeSystem()
+     * \sa Daemon::upgradeSystem()
+      */
+    enum UpgradeKind {
+        UpgradeKindUnknown,
+        UpgradeKindMinimal,
+        UpgradeKindDefault,
+        UpgradeKindComplete
+    };
+    Q_ENUM(UpgradeKind)
+
+    /**
+     * Describes the type of distribution upgrade to perform
+     * \sa Daemon::upgradeSystem()
      */
     enum TransactionFlag {
-        TransactionFlagNone         = 1 << 0, // Since: 0.8.1
-        TransactionFlagOnlyTrusted  = 1 << 1, // Since: 0.8.1
-        TransactionFlagSimulate     = 1 << 2, // Since: 0.8.1
-        TransactionFlagOnlyDownload = 1 << 3  // Since: 0.8.1
+        TransactionFlagNone           = 1 << 0, // Since: 0.8.1
+        TransactionFlagOnlyTrusted    = 1 << 1, // Since: 0.8.1
+        TransactionFlagSimulate       = 1 << 2, // Since: 0.8.1
+        TransactionFlagOnlyDownload   = 1 << 3,  // Since: 1.0.2
+        TransactionFlagAllowReinstall = 1 << 4,  // Since: 1.0.2
+        TransactionFlagJustReinstall  = 1 << 5,  // Since: 1.0.2
+        TransactionFlagAllowDowngrade = 1 << 6  // Since: 0.8.1
     };
     Q_DECLARE_FLAGS(TransactionFlags, TransactionFlag)
+    Q_ENUM(TransactionFlag)
 
     /**
      * Describes a restart type
@@ -382,6 +373,7 @@ public:
         RestartSecuritySession, /* a library that is being used by this package has been updated for security */
         RestartSecuritySystem
     };
+    Q_ENUM(Restart)
 
     /**
      * Describes an update's state
@@ -392,6 +384,7 @@ public:
         UpdateStateUnstable,
         UpdateStateTesting
     };
+    Q_ENUM(UpdateState)
 
     /**
      * Describes the different package groups
@@ -434,6 +427,7 @@ public:
         GroupNewest
     };
     typedef Bitfield Groups;
+    Q_ENUM(Group)
 
     /**
      * Describes the state of a package
@@ -463,8 +457,10 @@ public:
         InfoPreparing,
         InfoDecompressing,
         InfoUntrusted,
-        InfoTrusted
+        InfoTrusted,
+        InfoUnavailable
     };
+    Q_ENUM(Info)
 
     /**
      * Describes a signature type
@@ -473,6 +469,7 @@ public:
         SigTypeUnknown,
         SigTypeGpg
     };
+    Q_ENUM(SigType)
 
     /**
      * Create a transaction object with transaction id \p tid
@@ -673,13 +670,6 @@ public:
      */
     static QString packageData(const QString &packageID);
 
-    /**
-     * Returns the package icon from the \p packageID
-     *
-     * @deprecated use Appstream to fetch icons
-     */
-    static QT_DEPRECATED QString packageIcon(const QString &packageID);
-
 Q_SIGNALS:
     void allowCancelChanged();
 
@@ -768,13 +758,6 @@ Q_SIGNALS:
      */
     void finished(PackageKit::Transaction::Exit status, uint runtime);
 
-    /**
-     * Conveys a message sent from the backend
-     *
-     * \p type is the type of the \p message
-     */
-    void message(PackageKit::Transaction::Message type, const QString &message);
-
     /**
      * Emitted when the transaction sends a new package
      */
@@ -842,20 +825,12 @@ protected:
      * \attention Make sure to call this method in inherited classes
      * otherwise no signals will be emitted
      */
-    virtual void connectNotify(const char *signal);
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-    virtual void connectNotify(const QMetaMethod &signal);
-#endif
+    void connectNotify(const QMetaMethod &signal) override;
 
     /**
-     * This method disconnects from DBus signals
      * \attention Make sure to call this method in inherited classes
-     * otherwise no signals will be disconnected
      */
-    virtual void disconnectNotify(const char *signal);
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-    virtual void disconnectNotify(const QMetaMethod &signal);
-#endif
+    void disconnectNotify(const QMetaMethod &signal) override;
 
     TransactionPrivate * const d_ptr;
 
@@ -874,14 +849,11 @@ private:
     friend class Daemon;
     Q_DECLARE_PRIVATE(Transaction)
     Q_DISABLE_COPY(Transaction)
-    Q_PRIVATE_SLOT(d_func(), void createTransactionFinished(QDBusPendingCallWatcher*))
-    Q_PRIVATE_SLOT(d_func(), void methodCallFinished(QDBusPendingCallWatcher*))
     Q_PRIVATE_SLOT(d_func(), void distroUpgrade(uint type, const QString &name, const QString &description))
     Q_PRIVATE_SLOT(d_func(), void details(const QVariantMap &values))
     Q_PRIVATE_SLOT(d_func(), void errorCode(uint error, const QString &details))
     Q_PRIVATE_SLOT(d_func(), void mediaChangeRequired(uint mediaType, const QString &mediaId, const QString &mediaText))
     Q_PRIVATE_SLOT(d_func(), void finished(uint exitCode, uint runtime))
-    Q_PRIVATE_SLOT(d_func(), void message(uint type, const QString &message))
     Q_PRIVATE_SLOT(d_func(), void Package(uint info, const QString &pid, const QString &summary))
     Q_PRIVATE_SLOT(d_func(), void ItemProgress(const QString &itemID, uint status, uint percentage))
     Q_PRIVATE_SLOT(d_func(), void RepoSignatureRequired(const QString &pid, const QString &repoName, const QString &keyUrl, const QString &keyUserid, const QString &keyId, const QString &keyFingerprint, const QString &keyTimestamp, uint type))
@@ -901,7 +873,6 @@ Q_DECLARE_METATYPE(PackageKit::Transaction::InternalError)
 Q_DECLARE_METATYPE(PackageKit::Transaction::Role)
 Q_DECLARE_METATYPE(PackageKit::Transaction::Error)
 Q_DECLARE_METATYPE(PackageKit::Transaction::Exit)
-Q_DECLARE_METATYPE(PackageKit::Transaction::Message)
 Q_DECLARE_METATYPE(PackageKit::Transaction::Status)
 Q_DECLARE_METATYPE(PackageKit::Transaction::MediaType)
 Q_DECLARE_METATYPE(PackageKit::Transaction::DistroUpgrade)
@@ -914,5 +885,6 @@ Q_DECLARE_METATYPE(PackageKit::Transaction::SigType)
 Q_DECLARE_METATYPE(PackageKit::Transaction::Filter)
 Q_DECLARE_METATYPE(PackageKit::Transaction::TransactionFlags)
 Q_DECLARE_METATYPE(PackageKit::Transaction::Filters)
+Q_DECLARE_METATYPE(PackageKit::Transaction::UpgradeKind)
 
 #endif
